clear; clc;

% Load Quantity data (Q3 and Q4 contain different gilts)

Q1 = dataset('XLSFile','Q_1694m3-1860m2.xlsx');
Q2 = dataset('XLSFile','Q_1860m3-1881m2.xlsx');
Q3 = dataset('XLSFile','Q_1881m1-1947m12.xlsx');
Q4 = dataset('XLSFile','Q_1900m11-1947m12.xlsx');
Q5 = dataset('XLSFile','Q_1946m1-2018m12.xlsx');
Q=[Q1;Q2(Q2.Year<1881,:);Q3;Q4;Q5(Q5.Year>1947,:)];

% Load characteristics of individual gilts

d = dataset('XLSFile','Specification.xlsx');

% Define period of interest and maximum maturity in data

startyear=1729; 
endyear=2018;
maxmaturity=80;

% Define variables to store results

Maturity_y=zeros(12*(endyear-startyear+1)-5,1);
Maturity_m=zeros(12*(endyear-startyear+1)-5,1);
Qmonth=zeros(12*(endyear-startyear+1)-5,maxmaturity*12);
Qshort=zeros(12*(endyear-startyear+1)-5,1);
Qmedium=zeros(12*(endyear-startyear+1)-5,1);
Qlong=zeros(12*(endyear-startyear+1)-5,1);
Qconsols=zeros(12*(endyear-startyear+1)-5,1);
Qultra=zeros(12*(endyear-startyear+1)-5,1);
Quultra=zeros(12*(endyear-startyear+1)-5,1);
Qtotal=zeros(12*(endyear-startyear+1)-5,1);

% Loop through years and months

for year=startyear:endyear
    
    if year==endyear
        endmonth=12; % Data ends 2018m12
    else
        endmonth=12;
    end
    
    for month=1:endmonth
        
        [year,month]
        
        % Store year and month
        
        Maturity_y((year-startyear)*12+month,:)=year;
        Maturity_m((year-startyear)*12+month,:)=month;
        
        % Convert current year and month to number
        
        Settle=datenum([year  month  eomday(year,month)  0  0  0]);
                
        % Select gilts for given month
        
        % Non index-linked gilts (some anomalies omitted)
        Qtemp=Q(Q.Year==year & Q.Month==month & Q.Coupon>0 & Q.ID>0 & Q.Q>0 ...
        & Q.ID~=2300 & Q.ID~=2700 ...
        & Q.ID~=31200 & Q.ID~=30800 & Q.ID~=26800 & Q.ID~=31000 ...
        & Q.ID~=28100 & Q.ID~=17300 & Q.ID~=15700 & Q.ID~=14400 ...
        & Q.ID~=13300 & Q.ID~=12700 & Q.ID~=13000 ...
        & Q.ID~=18400 & Q.ID~=19600 & Q.ID~=31500 & Q.ID~=31800 ...
        & Q.ID<32300,:);
        % Index-linked gilts and some old stuff
        Qtemp2=Q(Q.Year==year & Q.Month==month & Q.Coupon>0 & Q.ID>0 & Q.Q>0 ...
        & Q.ID>49999 & Q.ID<100000,:);
        % Consols 
        Qtemp3=Q(Q.Year==year & Q.Month==month & Q.Coupon>0 & Q.ID>0 & Q.Q>0 ...
        & Q.ID>32299 & Q.ID<50000,:);
        % Miscellaneous old Consols
        Qtemp4=Q(Q.Year==year & Q.Month==month & Q.Coupon>0 & Q.ID>0 & Q.Q>0 ...
        & Q.ID>=100000,:);
        Qtemp=[Qtemp;Qtemp2;Qtemp3;Qtemp4];
        
        % Collect information for each bond in Qtemp

        % Variables for storage
        n=size(Qtemp,1);
        Maturity=zeros(n,1);
        Qs=zeros(n,1);
        c=zeros(n,1);
        
        % Store ID
        ID=Qtemp.ID;
        
        % Loop through gilts in Qtemp
        for i=1:n
            Qs(i)=Qtemp.Q(Qtemp.ID==ID(i),:);
            if (ID(i)>32299) && (ID(i)<50000) % Consols assigned maximum maturity
                Maturity(i)=datenum([year+maxmaturity-5  month  eomday(year+maxmaturity-5,month)  0  0  0]);
            elseif (ID(i)>100000) % Consols assigned maximum maturity
                Maturity(i)=datenum([year+maxmaturity-5  month  eomday(year+maxmaturity-5,month)  0  0  0]);
            else
                Maturity(i)=datenum(d.Redemption(d.ID==ID(i),:),'dd/mm/yyyy');
            end
            c(i)=Qtemp.Coupon(Qtemp.ID==ID(i),:); % Coupon
        end

        % Remove bonds with <= 0 days to redemption

        j=1;
        for i=1:n;
            if Maturity(i)-Settle>0;
                Qs(j)=Qs(i);
                ID(j)=ID(i);
                Maturity(j)=Maturity(i);
                c(j)=c(i);
                j=j+1;
            end
        end
        n=j-1;
        
        % Calculate maturity structure of outstanding debt
        
        % Variables for storage
        Qh=zeros(365*maxmaturity,1);
        Qh_nc=zeros(365*maxmaturity,1);
        Qhm=zeros(maxmaturity*12,1);
        Qhm_nc=zeros(maxmaturity*12,1);
        Qha=zeros(maxmaturity,1);
        Qha_nc=zeros(maxmaturity,1);
        
        % Loop through gilts in Qtemp
        for i=1:n
            Qh(Maturity(i)-Settle)=Qh(Maturity(i)-Settle)+Qs(i); % Redemption payment
            Qh_nc(Maturity(i)-Settle)=Qh_nc(Maturity(i)-Settle)+Qs(i); % Redemption payment
            for j=Maturity(i):-365/2:Settle+1
                Qh(round(j-Settle))=Qh(round(j-Settle))+c(i)*Qs(i)/200; % Add coupon
            end
        end  
        
        % Put maturity structure into monthly bins 
        
        for i=1:round((max(Maturity)-Settle)*12/365)+1
             Qhm(i)=sum(Qh(1+round((i-1)*(365/12)):round(i*(365/12))));
             Qhm_nc(i)=sum(Qh_nc(1+round((i-1)*(365/12)):round(i*(365/12))));
        end
        Qmonth((year-startyear)*12+month,:)=Qhm';
        Qmonth_nc((year-startyear)*12+month,:)=Qhm_nc';
        
        % Put monthly maturity structure into <1/ULTRA/SHORT/MEDIUM/LONG/CONSOL bins 

        Qtotal((year-startyear)*12+month,:)=sum(Qh_nc);
        Quultra((year-startyear)*12+month,:)=sum(Qh_nc(1:1*365));
        Qultra((year-startyear)*12+month,:)=sum(Qh_nc(1*365+1:3*365)); 
        Qshort((year-startyear)*12+month,:)=sum(Qh_nc(3*365+1:7*365));
        Qmedium((year-startyear)*12+month,:)=sum(Qh_nc(7*365+1:15*365));
        Qlong((year-startyear)*12+month,:)=sum(Qh_nc(15*365+1:74*365));
        Qconsols((year-startyear)*12+month,:)=sum(Qh_nc(74*365+1:end));
        
        % Put monthly maturity structure into annual bins
     
        for i=1:maxmaturity-1
            Qha(i)=sum(Qh(1+(i-1)*round(365):i*round(365)));
            Qha_nc(i)=sum(Qh_nc(1+(i-1)*round(365):i*round(365)));
        end
        Qmaturea((year-startyear)*12+month,:)=Qha;
        Qmaturea_nc((year-startyear)*12+month,:)=Qha_nc;
        
        % Plot maturity histogram to show progress
        
        figure(1);
        clf;
        bar(Qmonth((year-startyear)*12+month,:),1);
        drawnow;
        
    end
end

% Figure 3: Percentage composition of UK government debt by maturity

figure(3);
Time=Maturity_y+Maturity_m/12;
bar(Time,[Quultra./Qtotal Qultra./Qtotal Qshort./Qtotal Qmedium./Qtotal Qlong./Qtotal Qconsols./Qtotal], 1, 'stack')
title('Percentage composition of UK government debt by maturity','FontSize',20);
ylabel('Share');
ylim([0 1]);
xlim([1888 2020]);
set(gca,'FontSize',24)
legend('< 1 year','Ultra short < 3 years','Short 3-7 years','Medium 8-15 years','Long > 15 years','Consols','Location','southoutside','Orientation','horizontal');

% Figure 4: Structure of UK government debt

figure(4);
Y=Time*ones(1,maxmaturity);
X=ones(size(Y,1),1)*(1:1:maxmaturity);
Qmaturea_bin=1-double(Qmaturea_nc>0);
surface(Y',X',Qmaturea_bin','EdgeColor','none');
title('Structure of UK government debt');
ylabel('Maturity');
xlim([1888 2020]);
set(gca,'FontSize',24);