clear;clc;clf;

% Penalty function on stiffness when fitting yield curve

L = 9.2;
S = -1;
mu = 1;
lambdafun = @(t) exp(L - (L-S)*exp(-t/mu));

% Load GDP and debt data

load data_BOE YBHA3 YBHA3_real debt_boe % GDP is 1700-2017, Debt is 1691-2016

% Load characteristics of individual gilts

d = dataset('XLSFile','Specification.xlsx');

% Load Price data, excluding index-linked

P1 = dataset('XLSFile','P_1729m7-1860m2.xlsx');
P2 = dataset('XLSFile','P_1860m3-1888m2.xlsx');
P3 = dataset('XLSFile','P_1881m1-1949m3.xlsx');
P4 = dataset('XLSFile','P_1949m1-1962m12.xlsx');
P5 = dataset('XLSFile','P_1963m1-1975m12.xlsx');
P6 = dataset('XLSFile','P_1976m1-2018m12.xlsx');
P=[P1;P2(P2.Year<1888,:);P3(P3.Year<1949,:);P4;P5;P6(P6.ID<60000,:)];

% Adjust 4.5% War Loan prices in run up to 1917 conversion

load 1917X WL45axiss ytrue Ptrue WL45P;

for m=7:12
    P0=P.P(P.ID==60030 & P.Year==1915 & P.Month==m,:);
    P.P(P.ID==60030 & P.Year==1915 & P.Month==m,:)=100*Ptrue(m-6);
end
for m=1:12
    P0=P.P(P.ID==60030 & P.Year==1916 & P.Month==m,:);
    P.P(P.ID==60030 & P.Year==1915 & P.Month==m,:)=100*Ptrue(m-6+12);
end
for m=1:3
    P0=P.P(P.ID==60030 & P.Year==1917 & P.Month==m,:);
    P.P(P.ID==60030 & P.Year==1915 & P.Month==m,:)=100*Ptrue(m-6+24);
end

% Adjust 3.5% and 4.5% War Loan prices in run up to 1932 conversion

load 1932X WL45axiss ytrue45 Ptrue45 ytrue5 Ptrue5  WL45P WL5P;

for m=1:10
    P0=P.P(P.ID==60030 & P.Year==1932 & P.Month==m,:);
    P.P(P.ID==60030 & P.Year==1932 & P.Month==m,:)=100*Ptrue45(m+48);
    P0=P.P(P.ID==60040 & P.Year==1932 & P.Month==m,:);
    P.P(P.ID==60040 & P.Year==1932 & P.Month==m,:)=100*Ptrue5(m+48);
end

% Define period of interest

start_year=1915;
end_year=2018;

% Define variables to store results

Yields=zeros(12*(end_year-start_year+1),960);
Prices=zeros(12*(end_year-start_year+1),960);

% Loop through years and months

for year=start_year:end_year
    
    if year==end_year
        end_month=12; % Data ends 2018m12
    else
        end_month=12;
    end
    
    for month=1:end_month
        
        % Construct Zero Coupon Yield Curve

        % Load price data for year and month. Many exclusions due to
        % irregular data or anomalies
        
        Ptemp=P(P.Year==year & P.Month==month & P.ID>0 & P.P>0 & P.Coupon>0 ...
        & P.ID~=9000 & P.ID~=10900 & P.ID~=15600 & P.ID~=12400 ...
        & P.ID~=23800 & P.ID~=25200 & P.ID~=25300 & P.ID~=24200 ...
        & P.ID~=24500 & P.ID~=28900 & P.ID~=27200 & P.ID~=21200 ...
        & P.ID~=94 & P.ID~=100 & P.ID~=1800 & P.ID~=1801 & P.ID~=2300 ...
        & P.ID~=6700 & P.ID~=2700 & P.ID~=15100 & P.ID~=21300 ...
        & P.ID~=30500 & P.ID~=25800 & P.ID~=29000 & P.ID~=25400 ...
        & P.ID~=30900 & P.ID~=31700 &P.ID~=28800 & P.ID~=18900 ...
        & P.ID~=30400 & P.ID~=26300 & P.ID~=29600 & P.ID~=27000 ...
        & P.ID~=30300 & P.ID~=19500 & P.ID~=15900 & P.ID~=23400 ...
        & P.ID~=16600 & P.ID~=15800 & P.ID~=15400 & P.ID~=14900 ...
        & P.ID~=24600 & P.ID~=29500 & P.ID~=20400 & P.ID~=17600 ... 
        & P.ID~=13900 & P.ID~=15200 & P.ID~=14500 & P.ID~=25900 ... 
        & P.ID~=20300 & P.ID~=29400 & P.ID~=27400 & P.ID~=23000 ...
        & P.ID~=12200 & P.ID~=11800 & P.ID~=20900 & P.ID~=11200 ...
        & P.ID~=10400 & P.ID~=10000 & P.ID~=11600 & P.ID~=12500 ...
        & P.ID~=9700 & P.ID~=12000 & P.ID~=8900 & P.ID~=22400 ...
        & P.ID~=7900 & P.ID~=13700 & P.ID~=28600 & P.ID~=26000 ...
        & P.ID~=7400 & P.ID~=10500 & P.ID~=21800 & P.ID~=9200 ...
        & P.ID~=17200 & P.ID~=5500 & P.ID~=19800 & P.ID~=5000 ...
        & P.ID~=6000 & P.ID~=3800 & P.ID~=20200 & P.ID~=1200 ...
        & P.ID~=00 & P.ID~=00 & P.ID~=00 & P.ID~=00 ...
        & P.ID~=00 & P.ID~=00 & P.ID~=00 & P.ID~=00 ...
        & P.ID~=31200 & P.ID~=30800 & P.ID~=26800 & P.ID~=31000 ...
        & P.ID~=28100 & P.ID~=17300 & P.ID~=15700 & P.ID~=14400 ...
        & P.ID~=13300 & P.ID~=12700 & P.ID~=13000 & P.ID~=11300 ...
        & P.ID~=18400 & P.ID~=19600 & P.ID~=31800 & P.ID<32300,:);

        Ptemp2=P(P.Year==year & P.Month==month & P.ID>0 & P.P>0 & P.Coupon>0 ...
        & P.ID>59999 & P.ID~=60071 & P.ID~=60072 & P.ID~=60081 & P.ID~=60082 ...
        & P.ID~=60600 & P.ID~=60370 &P.ID~=60090 &P.ID~=60150 &P.ID~=60160 ...
        & P.ID~=60180 & P.ID~=60190 &P.ID~=60200 &P.ID~=60210 &P.ID~=60230 &P.ID~=60020 &P.ID~=60080,:);

        Ptemp=[Ptemp;Ptemp2];
        
        % Data on each gilt traded in market in current year and month
        
        % Number of gilts traded
        n=size(Ptemp,1); 
        % Current settlement date
        CurveSettle=datenum([year  month  eomday(year,month)  0  0  0]);
        Settle=ones(n,1)*CurveSettle;
        % Current price
        CleanPrice=Ptemp.P;
        % Coupon rate
        CouponRate=Ptemp.Coupon/100;
        % ID number
        ID=Ptemp.ID;

        % Look up redemption date of each gilt
        
        Maturity=zeros(n,1);
        for i=1:n
            Maturity(i)=datenum(d.Redemption(d.ID==ID(i),:),'dd/mm/yyyy');
        end
        
        % Only select gilts with at least 3 months until redemption

        j=1;
        for i=1:n
            if Maturity(i)-CurveSettle>93
                Settle(j)=Settle(i);
                CleanPrice(j)=CleanPrice(i);
                CouponRate(j)=CouponRate(i);
                ID(j)=ID(i);
                Maturity(j)=Maturity(i);
                j=j+1;
            end
        end

        % Collect data for estimation
        
        Settle=Settle(1:j-1);
        CleanPrice=CleanPrice(1:j-1);
        CouponRate=CouponRate(1:j-1);
        ID=ID(1:j-1);
        Maturity=Maturity(1:j-1);    
        n=size(Maturity,1);
        Instruments = [Settle Maturity CleanPrice CouponRate];
        InstrumentPeriod = [repmat(2,n,1)];

        % Fit zero coupon yield curve using Financial Instruments Toolbox

        CustomKnots = augknt(0:5:50,4);
        VRPModel = IRFunctionCurve.fitSmoothingSpline('Forward',CurveSettle,...
                   Instruments,lambdafun,'Compounding',-1,...
                   'InstrumentPeriod',InstrumentPeriod,'Knots',CustomKnots);

        % Calculate yields to plot     
        
        PlottingDates = CurveSettle+20:30:CurveSettle+960*30;
        TimeToMaturity = yearfrac(CurveSettle,PlottingDates);
        VRPZeroRates = VRPModel.getZeroRates(PlottingDates);
        Yield = bndyield(CleanPrice, CouponRate, Settle, Maturity);
        
        % Plot estimated yield curve and market data
 
        clf
        h2=figure(1);
        hold on
        scatter(Maturity,Yield); % Data
        text(Maturity,Yield,num2str(ID));
        PlottingPoints = min(Maturity):30:max(Maturity);
        hold on;
        plot(PlottingPoints, getParYields(VRPModel, PlottingPoints),'r','LineWidth',2); % Estimates
        datetick
        sp={' '};
        name=strcat('Fit to market data for', sp, num2str(year), ':', num2str(month));
        title(name);
        drawnow;
        
        % Save figure to file
        
        %name=strcat('fit_', num2str(year), '_', num2str(month));
        %saveas(h2,name,'eps')
   
        % Store estimated zero coupon yields and prices
        
        Yields((year-start_year)*12+month,:)=VRPZeroRates';
        Prices((year-start_year)*12+month,:)=bndprice(VRPZeroRates,0,Settle(1),PlottingDates)';
         
    end
    
end

% Calculate inflation and (annual) ex post real yield

GDP_deflator1=YBHA3(1915-1700:end-1)./YBHA3_real(1915-1700:end-1);
GDP_deflator2=YBHA3(1915-1700+1:end)./YBHA3_real(1915-1700+1:end);
inflation=100*((GDP_deflator2-GDP_deflator1)./GDP_deflator1);
Yields_r=Yields(1:12:1236,12:12:30*12)-0.01*inflation*ones(1,30);
nominal_index=(1915:1/12:2018+11/12)'*ones(1,365);
real_index=(1915:1:2017)'*ones(1,365);

% Save yields and prices

save ZCY_data.mat Yields Yields_r Prices nominal_index real_index;

% Figure 7: Selected zero coupon yields

figure(7);
subplot(2,2,1);
plot(nominal_index,Yields(:,1),real_index,Yields_r(:,1),'LineWidth',2);
legend('Nominal','Real','Location','southeast','AutoUpdate','off');
line([1910 2017],[0 0],'LineWidth',2,'Color','k');
xlim([1910 2020]);
ylim([-0.2 0.2]);
set(gca,'FontSize',20);
title('Yield on 1-year bond');
set(gca,'FontSize',20);

subplot(2,2,2);
plot(nominal_index,Yields(:,36),real_index,Yields_r(:,3),'LineWidth',2);
legend('Nominal','Real','Location','southeast','AutoUpdate','off');
hold on;
line([1910 2017],[0 0],'LineWidth',2,'Color','k');
xlim([1910 2020]);
ylim([-0.2 0.2]);
title('Yield on 3-year bond');
set(gca,'FontSize',20);

subplot(2,2,3);
plot(nominal_index,Yields(:,120),real_index,Yields_r(:,10),'LineWidth',2);
legend('Nominal','Real','Location','southeast','AutoUpdate','off');
hold on;
line([1910 2017],[0 0],'LineWidth',2,'Color','k');
xlim([1910 2020]);
ylim([-0.2 0.2]);
title('Yield on 10-year bond');
set(gca,'FontSize',20);

subplot(2,2,4);
plot(nominal_index,Yields(:,120)-Yields(:,36),'LineWidth',2);
hold on;
line([1910 2017],[0 0],'LineWidth',2,'Color','k');
xlim([1910 2020]);
ylim([-0.04 0.06]);
title('Term premium');
set(gca,'FontSize',20);