function val=fixed3(X)

global Yields Prices YBHA3 YBHA3_real debt_boe startyearGDP endyearGDP;
global Qmonthi startyeari endyeari;

% Load yields and prices for counterfactuals

load ZCY_data.mat Yields Yields_r Prices nominal_index real_index

% Load nominal payments on nominal fixed-term debt at monthly frequency

load returns_f_data index_for_pay pay pay1 pay2;
 
% Load face and market values of debt at monthly frequency

load debt_f_data debt_face debt_face_n debt_face_i debt_market ...
     debt_market_n debt_market_i startyear endyear;
 
% Load GDP data

load data_BOE YBHA3 YBHA3_real debt_boe % Debt is 1691-2016, GDP is 1700-2016
startyearGDP=1700; endyearGDP=2017;

% Load issuance data

load issuance_f_data Qmonthi startyeari endyeari;

% Construct VaR measure at horizon fh

fh=1;VCV=cov(Prices(1+fh:end,:)-Prices(1:end-fh,:));

% Market and face value of issuance

Market_value_issuance=Prices.*Qmonthi;
Face_value_issuance=100.*Qmonthi;

% Read in function paramaters. h1 and h2 are horizons of counterfactuals,
% z is proportion, x is issuance cost

h1=X(1);
h2=X(2);
h3=X(3);
z1=X(4);
z2=X(5);

% Construct actual evolution of face value of debt-GDP and implied cash flow

n=12*(endyeari-startyeari+1);
mm=size(Prices,2);
debt=zeros(n,mm);
cash=zeros(n+mm,1);
VaR=zeros(n+mm,1);

for i=2:n
     debt(i,:)=[debt(i-1,2:end) 0]+Face_value_issuance(i,:);
     VaR(i)=norminv(0.05)*(0.01^2*Face_value_issuance(i,:)*VCV*Face_value_issuance(i,:)')^0.5;
     cash(i)=cash(i)+sum(Market_value_issuance(i,:));
     cash(i+1:i+mm)=cash(i+1:i+mm)-Face_value_issuance(i,:)';
end

% Construct counterfactual evolution when always issue h year bonds

cash_c=cash; % Initial funding requirement taken from data

Face_value_issuance_c=zeros(n,mm);
Market_value_issuance_c=zeros(n,mm);
debt_c=zeros(n,mm);
VaR_c=zeros(n+mm,1);

for i=2:n
    
    % Only issue bonds if government needs cash
    
    if cash_c(i)>0
        Market_value_issuance_c(i,h1)=z1*cash_c(i);
        Market_value_issuance_c(i,h2)=Market_value_issuance_c(i,h2)+z2*cash_c(i);
        Market_value_issuance_c(i,h3)=Market_value_issuance_c(i,h3)+(1-z1-z2)*cash_c(i);
        Face_value_issuance_c(i,h1)=z1*cash_c(i)*(100/Prices(i,h1));
        Face_value_issuance_c(i,h2)=Face_value_issuance_c(i,h2)+z2*cash_c(i)*(100/Prices(i,h2));
        Face_value_issuance_c(i,h3)=Face_value_issuance_c(i,h3)+(1-z1-z2)*cash_c(i)*(100/Prices(i,h3));
        cash_c(i+h1)=cash_c(i+h1)+z1*cash_c(i)*(100/Prices(i,h1));
        cash_c(i+h2)=cash_c(i+h2)+z2*cash_c(i)*(100/Prices(i,h2));
        cash_c(i+h3)=cash_c(i+h3)+(1-z1-z2)*cash_c(i)*(100/Prices(i,h3));
    else
        cash_c(i+1)=cash_c(i+1)+cash_c(i);
    end
    
    debt_c(i,:)=[debt_c(i-1,2:end) 0]+Face_value_issuance_c(i,:); % Evolution of face value of debt
    VaR_c(i,:)=norminv(0.05)*(0.01^2*Face_value_issuance_c(i,:)*VCV*Face_value_issuance_c(i,:)')^0.5; % VaR

end

% Extract annual data

debt_mv_c=0.01*Prices(end-size(debt,1)+1:end,:).*debt_c;
debt_mv_a_c=zeros((n-12)/12,1);
VaR_a_c=zeros((n-12)/12,1);

for i=1:(n-12)/12
	debt_mv_a_c(i)=sum(mean(debt_mv_c((i-1)*12+1:i*12,:),1))./YBHA3(i+startyeari-startyearGDP); 
    VaR_a_c(i)=mean(VaR_c((i-1)*12+1:i*12))./YBHA3(i+startyeari-startyearGDP);
end

% Temporarily save face value of debt/GDP and actual and counterfactual cash needs

save tempsave1 debt_c cash cash_c;

% Return horizons, average debt and VaR

val=[h1 h2 h3 mean(debt_mv_a_c) -mean(VaR_a_c)];